      subroutine gradconv (ncmpct,pcmpct,atol,rtol,isconv)
c
c=======================================================================
c                                                                    ===
c  This routine examines the current pressure tendency to see if its ===
c  gradients have converged to within the requested tolerences.      ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     NCMPCT   Number of active pressure points.          (integer)  ===
c     PCMPCT   Surface pressure change, compact storage.  (dble prec)===
c     ATOL     Convergence absolute tolerence.            (dble prec)===
c     RTOL     Convergence relative tolerence.            (dble prec)===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
#ifdef coast
c  /FULLWD/                                                          ===
c                                                                    ===
c     IEP   Ending indices for surface pressure.     (integer array) ===
c     IEQ   Ending indices for velocity points.      (integer array) ===
c     ISP   Starting indices for surface pressure.   (integer array) ===
c     ISQ   Starting indices for velocity points.    (integer array) ===
c                                                                    ===
#endif
c  /ONEDIM/                                                          ===
c                                                                    ===
c     CSR     reciprocal of cosine, velocity points.   (real vector) ===
c     DXU2R   reciprocal of twice velocity x-spacing.  (real vector) ===
c     DYU2R   reciprocal of twice velocity y-spacing.  (real vector) ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     ISCONV   Flag to indicate convergence.               (logical) ===
c              [true]   gradients have converged.                    ===
c              [false]  gradients have not yet converged.            ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#ifdef coast
# include <fullwd.h>
#endif
#include <onedim.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iend,istr,j,knew,kold,n,ncmpct
#ifdef coast
      integer l
#endif
      logical first,isconv
      double precision 
     &      atol,rtol,testval
      double precision 
     &      dcsr(jmt),ddxu2r(imt),ddyu2r(jmt),dpsdt(imt,jmt),
     &      d2psdxdt(imt,jmt,0:1),d2psdydt(imt,jmt,0:1),pcmpct(ncmpct)
c
      save first,knew,kold,dcsr,ddxu2r,ddyu2r,d2psdxdt,d2psdydt
c
      data knew,kold /0,1/
      data first /.true./
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Transfer change in surface pressure from compact storage to 2D.
c-----------------------------------------------------------------------
c
      n    = 0
      istr = 2
      iend = imtm1
c
      do 30 j = 2, jmtm1
# ifdef coast
         do 20 l = 1, lseg
            istr = isp(j,l)
            if(istr.eq.0) go to 30
            iend = iep(j,l)
# endif
            do 10 i = istr, iend
               n = n+1
               dpsdt(i,j)=pcmpct(n)
  10        continue
# ifdef coast
  20     continue
# endif
  30  continue
c
c-----------------------------------------------------------------------
c  Examine change in gradients of surface pressure tendency.
c-----------------------------------------------------------------------
c
c  Rotate storage pointers.
c
      knew = 1 - knew
      kold = 1 - knew
      testval = c0
      istr = 2
      iend = imtm2
c
c  Loop over all active velocity points, examining gradient.
c
      do 60 j = 2, jmtm2
#ifdef coast
         do 50 l=1,lseg
            istr=isq(j,l)
            if(istr.eq.0) go to 60
            iend=ieq(j,l)
#endif
            do 40 i = istr, iend
c
               d2psdxdt(i,j,knew) = ((dpsdt(i+1,j+1)-dpsdt(i,j+1)) +
     &                               (dpsdt(i+1,j)-dpsdt(i,j)))
     &                              *dcsr(j)*ddxu2r(i)
c
               d2psdydt(i,j,knew) = ((dpsdt(i+1,j+1)-dpsdt(i+1,j)) +
     &                               (dpsdt(i,j+1)-dpsdt(i,j)))
     &                              *ddyu2r(j)
c
               testval = max(testval,
     &                       (abs(d2psdxdt(i,j,knew)-d2psdxdt(i,j,kold))
     &                        -(rtol*abs(d2psdxdt(i,j,knew))+atol)),
     &                       (abs(d2psdydt(i,j,knew)-d2psdydt(i,j,kold))
     &                        -(rtol*abs(d2psdydt(i,j,knew))+atol)) )
c
  40        continue
#ifdef coast
  50     continue
# endif
  60  continue
c
      isconv = testval .le. c0
c
      return
c
c-----------------------------------------------------------------------
c  Entry to initialize rolling storage.
c-----------------------------------------------------------------------
c
      entry gcinit
c
      do 90 n = 0, 1
         do 80 j = 1, jmt
            do 70 i = 1, imt
               d2psdxdt(i,j,n) = c0
               d2psdydt(i,j,n) = c0
  70        continue
  80     continue
  90  continue
c
      if (first) then
         do 100 i = 1, imt
            ddxu2r(i) = dble(dxu2r(i))
 100     continue
         do 110 j = 1, jmt
            dcsr(j)   = dble(csr(j))
            ddyu2r(j) = dble(dyu2r(j))
 110     continue
         first = .false.
      end if
c
      return
      end
