      subroutine rprfpos (keytv,htype,nbprf,ipos,bgnptr,emtptr)
c
c=======================================================================
c                                                                    ===
c  This routine reads the profile positions and times for a single   ===
c  instrument.                                                       ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     KEYTV    Flag for tracer or velocity grid.    (integer)        ===
c     HTYPE    Instrument name.                     (string)         ===
c     NBPRF    Number of profiles to sample.        (integer)        ===
c                                                                    ===
c   Common Blocks:                                                   ===
c                                                                    ===
c   /FULLWD/                                                         ===
c                                                                    ===
#ifndef resetjulian
c     DSTART    Initialization starting time.    (real; mod Jul day) ===
#else
c     D0START   Reset starting time.             (real; mod Jul day) ===
#endif
c     NLAST     Number of timesteps to compute.   (integer)          ===
c                                                                    ===
c   /IOUNITS/                                                        ===
c                                                                    ===
c     INPPRF   Unit number for sampling input file.   (integer)      ===
c     STDOUT   Standard output logical unit.          (integer)      ===
c                                                                    ===
c   /MODDAT/                                                         ===
c                                                                    ===
c     COORD    type of horizontal coordinates.      (integer)        ===
c                 [0] cartesian                                      ===
c                 [1] spherical                                      ===
c                 [2] rotated spherical                              ===
c     GRIDX    grid spacing in the x-direction.     (real; cm | deg) ===
c     GRIDY    grid spacing in the y-direction.     (real; cm | deg) ===
c     RLATD    transformation centroid longitude.   (real; degrees)  ===
c     RLOND    transformation centroid latitude.    (real; degrees)  ===
c     DELX     domain "longitudinal" shift.         (real; cm | deg) ===
c     DELY     domain "latitudinal" shift.          (real; cm | deg) ===
c     THETAD   domain rotation angle.               (real; degrees)  ===
c                                                                    ===
c   /SCALAR/                                                         ===
c                                                                    ===
c     DTTS     length of timestep on tracer.   (real; s)             ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     IPOS     Index profile positions.             (integer array)  ===
c     BGNPTR   Pointer to start of position list.   (integer)        ===
c     EMTPTR   Pointer to empty portion of list.    (integer)        ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     EXITUS,  LL2XY,  NO_DIGIT                                      ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <fullwd.h>
#include <iounits.h>
#include <moddat.h>
#include <scalar.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer bgnptr,emtptr,ios,keytv,n,nbprf,nkeep,itime,ix,iy
      integer ipos(0:mnbprf,4)
      integer no_digit
      FLOAT
     &      dstarf,rdtime,x,xyoff,y
      FLOAT
     &              lat(mnbprf),lon(mnbprf),time(mnbprf)
      character*128 fmt
      character*(*) htype
c
c=======================================================================
c  Begin excutable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Read position data.
c-----------------------------------------------------------------------
c
      read (inpprf,*,iostat=ios) (lat(n),lon(n),time(n) ,n=1,nbprf)
c
      if (ios.gt.0) then
         write (stdout,900) 'error',htype
         call exitus ('RPRFPOS')
      else if (ios.lt.0) then
         write (stdout,900) 'end of file',htype
         call exitus ('RPRFPOS')
      endif
c
c-----------------------------------------------------------------------
c  Convert to nearest indices.  Eliminate out-of-bounds positions.
c-----------------------------------------------------------------------
c
c  Set counter, grid offset, absolute time and reciprocal time-step.
c
      nkeep = 0
c
      if (keytv.ne.vgrid) then
         xyoff = c0
       else
         xyoff = -p5
      endif
c
      rdtime = day2sec/dtts
c
#ifndef resetjulian
      dstarf = dstart
#else
      dstarf = d0start
#endif
c
c  Loop over all requested positions.  Convert to (i,j,itt) indices.
c  Eliminate out-of-bounds values.
c
      do 10 n = 1, nbprf
         call ll2xy (lon(n),lat(n),coord,imt,jmt,gridx,gridy,rlngd,
     &                                       rlatd,delx,dely,thetad,x,y)
         ix    = nint(x+xyoff)
         iy    = nint(y+xyoff)
         itime =  nint((time(n)-dstarf)*rdtime) + 1
c
         if ((ix.ge.1) .and. (ix.le.imt) .and. (iy.ge.1) .and.
     &       (iy.le.jmt) .and. (itime.ge.1) .and. (itime.le.nlast)) then
            nkeep = nkeep + 1
            ipos(nkeep,1) = ix
            ipos(nkeep,2) = iy
            ipos(nkeep,3) = itime
         endif
  10  continue
c
      if (nkeep.lt.nbprf) then
         write (fmt,910) no_digit (nbprf-nkeep)
         write (stdout,fmt) (nbprf-nkeep),htype
      endif
c
c  Update profile count and pointer data.
c
      nbprf = nkeep
      ipos(nbprf,4) = 0
      bgnptr = 1
      emtptr = nbprf+1
c
      return
c
 900  format (/'***Error:  RPRFPOS - ',a,' while reading data for ',
     &         'instrument ',a)
 910  format ('(/'' RPRFPOS - eliminated '',i',i2.2,
     &        ','' out-of-bounds positions for instrument '',a)')
c
      end
